# -*- coding: utf-8 -*-

from plot import *

minfloat = 4.9406564584124654e-324
maxfloat = 8.9884656743115795e+307

def plotError ( name, v, N, case='mean', frame=False, color=None, refr=None, K=1, infolines=False, save=None ):
  '''Plots the errors for the refinements in the list N'''
  
  reft = len(refr.times)-1
  refd = refr.var(v, reft, case)
  
  errors = []
  ncells = []
  rtimes = []
  
  for i , (NX, NY, NZ) in enumerate(N):
    
    refdd = refd
    
    zoom = refr.i.NX/NX
    if refr.i.NY == 1:
      while zoom > 1:
        refdd = (refdd[0:refr.i.NX-1:2,:,:]+refdd[1:refr.i.NX:2,:,:])/2
        zoom /= 2
    elif refr.i.NZ == 1:
      while zoom > 1:
        refdd = (refdd[0:refr.i.NX-1:2,0:refr.i.NY-1:2,:]+refdd[1:refr.i.NX:2,0:refr.i.NY-1:2,:]+refdd[1:refr.i.NX:2,1:refr.i.NY:2,:]+refdd[0:refr.i.NX-1:2,1:refr.i.NY:2,:])/4
        zoom /= 2
    else:
      while zoom > 1:
        refdd = (  refdd[0:refr.i.NX-1:2, 0:refr.i.NY-1:2, 0:refr.i.NZ-1:2] \
                 + refdd[0:refr.i.NX-1:2, 0:refr.i.NY-1:2, 1:refr.i.NZ:2  ] \
                 + refdd[0:refr.i.NX-1:2, 1:refr.i.NY:2,   0:refr.i.NZ-1:2] \
                 + refdd[0:refr.i.NX-1:2, 1:refr.i.NY:2,   1:refr.i.NZ:2  ] \
                 + refdd[1:refr.i.NX:2,   0:refr.i.NY-1:2, 0:refr.i.NZ-1:2] \
                 + refdd[1:refr.i.NX:2,   0:refr.i.NY-1:2, 1:refr.i.NZ:2  ] \
                 + refdd[1:refr.i.NX:2,   1:refr.i.NY:2,   0:refr.i.NZ-1:2] \
                 + refdd[1:refr.i.NX:2,   1:refr.i.NY:2,   1:refr.i.NZ:2  ] ) / 8
        zoom /= 2
    
    name_base = name + '_%dx%dx%d' % (NX, NY, NZ)
    
    error = 0
    rtime = 0
    for k in range(1, K+1):
      r = Run('%s-%d' % (name_base, k))
      
      t = len(r.times)-1
      d = r.var(v, t, case)
      
      error += (100*L1(d, refdd))**2
      
      if 'wall_runtime_total' in r.runtime:
        rtime += r.runtime['wall_runtime_total']
      else:
        rtime += r.runtime['clock_runtime']
    
    errors.append ( numpy.sqrt(error/K) )
    ncells.append ( NX )
    rtimes.append ( rtime/K )
  
  # plot error vs ncells
  
  pylab.subplot(121)
  pylab.plot(numpy.log10(ncells), numpy.log10(errors), '-x', label=name, color=color)
  pylab.title('Relative L1 error of ' + case + ' of ' + v.name + ' (K=%d)' % K)
  pylab.ylabel(r'log10($L^2(\Omega,\ L^1)$ - error')
  pylab.xlabel('log10(cells) in x-direction')
  pylab.legend(loc='lower left')
  pylab.grid(True)
  
  slopes = [[1,2], [1,1]]
  if not frame:
    for slope in slopes:
      plotSlope(slope, pylab.gca())
  
  # plot error vs runtime
  
  pylab.subplot(122)
  pylab.plot(numpy.log10(rtimes), numpy.log10(errors), '-x', label=name, color=color)
  pylab.title('Relative L1 error of ' + case + ' of ' + v.name + ' (K=%d)' % K)
  pylab.ylabel(r'log10($L^2(\Omega,\ L^1)$ - error')
  pylab.xlabel('log10(seconds)')
  pylab.legend(loc='lower left')
  pylab.grid(True)
  
  dim = int(refr.i.NX>1) + int(refr.i.NY>1) + int(refr.i.NZ>1)
  
  slopes = [[1,2+2*dim], [1,1+dim]]
  if not frame:
    for slope in slopes:
      plotSlope(slope, pylab.gca())
  
  pylab.subplots_adjust(right=0.98)
  pylab.subplots_adjust(left=0.06)
  pylab.subplots_adjust(top=0.94)
  
  if infolines: refr.showInfo('Reference ')
  
  if not frame:
    pylab.show()
  if save:
    pylab.savefig(save)

# plots slopes
def plotSlope(slope, plt):
  '''Plots the slope together with a numerical value in the plt axis'''
  
  from pylab import xlim, ylim, loglog
  
  xmin, xmax = xlim()
  ymin, ymax = ylim()
  
  x1 = xmax - (xmax-xmin)/3
  x2 = xmax - (xmax-xmin)/15
  
  y1 = ymax - (ymax-ymin)/20
  y2 = y1 - float(slope[0])/slope[1] * (x2 - x1)
  
  plt.plot([x1, x2], [y1, y2], '--k', label='_nolegend_')
  plt.annotate(' %d/%d' % (slope[0], slope[1]), xy=(x2, y2),  xycoords='data')

def plotEfficiency ( name, N, frame=False, color=None, K=1, save=None ):
  '''Plots the parallelisation efficiency for the refinements in the list N'''
  
  ncells = []
  
  clock_runtimes          = []
  wall_runtimes_total     = []
  mpi_wall_runtimes_total = []
  mpi_scalings            = []
  
  for i, (NX, NY, NZ) in enumerate(N):
    
    name_base = name + '_%dx%dx%d' % (NX, NY, NZ)
    
    clock_runtime          = 0
    wall_runtime_total     = 0
    mpi_wall_runtime_total = 0
    mpi_scaling            = 0
    
    for k in range(1, K+1):
      container = {}
      execfile('%s_%s-%d.py' % ('runtime', name_base, k), globals(), container)
      clock_runtime          += container['clock_runtime']
      wall_runtime_total     += container['wall_runtime_total']
      mpi_wall_runtime_total += container['mpi_wall_runtime_total']
      mpi_scaling            += container['mpi_scaling']
    
    ncells.append ( NX * NY * NZ )
    
    clock_runtimes          .append ( clock_runtime/K )
    wall_runtimes_total     .append ( wall_runtime_total/K )
    mpi_wall_runtimes_total .append ( mpi_wall_runtime_total/K )
    mpi_scalings            .append ( mpi_scaling/K )
  
  # runtimes vs ncells
  
  pylab.subplot(121)
  pylab.plot(numpy.log10(ncells), numpy.log10(wall_runtimes_total), '-x', label=name, color=color)
  pylab.plot(numpy.log10(ncells), numpy.log10(mpi_wall_runtimes_total), color+'--')
  pylab.title('Runtime: Wall time vs. MPI calls wall time | average of %d runs' % K)
  pylab.ylabel('log10(seconds)')
  pylab.xlabel('log10(cells)')
  pylab.legend(loc='upper left')
  pylab.grid(True)
  
  # MPI efficiency vs ncells
  
  pylab.subplot(122)
  pylab.plot(numpy.log10(ncells), mpi_scalings, '-x', label=name, color=color)
  pylab.ylim(0,1)
  pylab.title('MPI efficiency: 1 - (MPI wall time) / (wall time) | average of %d runs' % K)
  pylab.ylabel('efficiency')
  pylab.xlabel('log10(cells)')
  pylab.legend(loc='lower right')
  pylab.grid(True)
  
  pylab.subplots_adjust(right=0.98)
  pylab.subplots_adjust(left=0.06)
  pylab.subplots_adjust(top=0.94)
  
  if not frame:
    pylab.show()
  if save:
    pylab.savefig(save)

def plotStrongScaling ( name, N, frame=False, color=None, K=1, save=None ):
  '''Plots the strong scaling for the refinements in the list N'''
  
  # === strong scaling
  
  wall_runtimes = []
  mpi_scalings  = []
  nodes         = []
  
  for i in range(1, len(N)+1):
    
    name_base = name + '_%d' % i
    
    wall_runtime = 0
    mpi_scaling  = 0
    
    for k in range(1, K+1):
      container = {}
      execfile('%s_%s-%d.py' % ('runtime', name_base, k), globals(), container)
      wall_runtime += container['wall_runtime']
      mpi_scaling  += container['mpi_scaling']
    
    wall_runtimes .append ( wall_runtime/K )
    mpi_scalings  .append ( mpi_scaling/K )
    nodes         .append ( container['nodes'] )
  
  pylab.subplot(121)
  pylab.plot(numpy.log10(nodes), numpy.log10(wall_runtimes), '-x', label=name, color=color)
  pylab.title('Strong scaling | average of %d runs' % K)
  pylab.ylabel('log10(wall runtime)')
  pylab.xlabel('log10(nodes)')
  pylab.legend(loc='lower left')
  pylab.grid(True)
  
  if not frame:
    plotSlope([1,1], pylab.gca())
  
  # === efficiency
  
  pylab.subplot(122)
  pylab.plot(numpy.log10(nodes), mpi_scalings, '-x', label=name, color=color)
  pylab.ylim(0,1)
  pylab.title('MPI efficiency | average of %d runs' % K)
  pylab.ylabel('efficiency')
  pylab.xlabel('log10(nodes)')
  pylab.legend(loc='upper left')
  pylab.grid(True)
  
  # === formatting
  
  pylab.subplots_adjust(right=0.98)
  pylab.subplots_adjust(left=0.06)
  pylab.subplots_adjust(top=0.94)
  
  if not frame:
    pylab.show()
  if save:
    pylab.savefig(save)

def plotWeakScaling ( name, N, frame=False, color=None, K=1, save=None ):
  '''Plots the weak scaling for the refinements in the list N'''
  
  # === weak scaling
  
  wall_runtimes_total = []
  mpi_scalings        = []
  nodes               = []
  
  for i, (NX, NY, NZ) in enumerate(N):
    
    name_base = name + '_%dx%dx%d' % (NX, NY, NZ)
    
    wall_runtime_total = 0
    mpi_scaling        = 0
    
    for k in range(1, K+1):
      container = {}
      execfile('%s_%s-%d.py' % ('runtime', name_base, k), globals(), container)
      wall_runtime_total += container['wall_runtime_total']
      mpi_scaling        += container['mpi_scaling']
    
    r = Run( '%s-%d' % (name_base, 1) )
    dim = int(r.i.NX>1) + int(r.i.NY>1) + int(r.i.NZ>1)
    order = int(r.vars['ORDER'])
    
    log_factor = 1
    if dim+1 == order:
      log_factor = ( i + numpy.log2(N[0][0]) ) / numpy.log2(N[0][0])
    
    wall_runtimes_total .append ( (wall_runtime_total/K) / ((2**(float(dim+1)/dim))**i * log_factor) )
    mpi_scalings        .append ( mpi_scaling/K )
    nodes               .append ( container['nodes'] )
  
  pylab.subplot(121)
  pylab.plot(numpy.log10(nodes), wall_runtimes_total, '-x', label=name, color=color)
  (ymin, ymax) = pylab.ylim()
  pylab.ylim(0,ymax)
  pylab.title('"Weak" scaling | average of %d runs' % K)
  pylab.ylabel('cumulative wall runtime / problem size')
  pylab.xlabel('log10(nodes)')
  pylab.legend(loc='upper left')
  pylab.grid(True)
  
  # === efficiency
  
  pylab.subplot(122)
  pylab.plot(numpy.log10(nodes), mpi_scalings, '-x', label=name, color=color)
  pylab.ylim(0,1)
  pylab.title('MPI efficiency | average of %d runs' % K)
  pylab.ylabel('efficiency')
  pylab.xlabel('log10(nodes)')
  pylab.legend(loc='upper left')
  pylab.grid(True)
  
  # === formatting
  
  pylab.subplots_adjust(right=0.98)
  pylab.subplots_adjust(left=0.06)
  pylab.subplots_adjust(top=0.94)
  
  if not frame:
    pylab.show()
  if save:
    pylab.savefig(save)

def plotStochasticScaling ( name, KL, frame=False, color=None, K=1, save=None ):
  '''Plots the stochastic scaling for the number of stochastic dimensions in the list KL'''
  
  rtimes = []
  
  for m in KL:
    
    if name == '':
      name_base = 'KL=%d' % m
    else:
      name_base = name + '_KL=%d' % m
    
    rtime = 0
    
    for k in range(1, K+1):
      container = {}
      execfile('%s_%s-%d.py' % ('runtime', name_base, k), globals(), container)
      if 'wall_runtime_total' in container:
        rtime += container['wall_runtime_total']
      else:
        rtime += container['clock_runtime']
    
    rtimes.append ( rtime/K )
  
  pylab.plot(numpy.log10(KL), numpy.log10(rtimes), '-x', label=name, color='k')
  pylab.title('Stochastic scaling | average of %d runs' % K)
  pylab.ylabel('log10(seconds)')
  pylab.xlabel('log10(KL-coefficients)')
  pylab.legend(loc='upper left')
  pylab.grid(True)
  
  if not frame:
    plotSlope([-1,1], pylab.gca())
  
  Run('%s-1' % name_base).showInfo()
  
  # === formatting
  
  pylab.subplots_adjust(right=0.96)
  pylab.subplots_adjust(left=0.10)
  pylab.subplots_adjust(top=0.94)
  
  if not frame:
    pylab.show()
  if save:
    pylab.savefig(save)

# helpers
make1d_N = lambda refs, start=32 : [[start*2**i,1,1] for i in range(refs)]
make2d_N = lambda refs, start=32 : [[start*2**i,start*2**i,1] for i in range(refs)]
make3d_N = lambda refs, start=32 : [[start*2**i,start*2**i,start*2**i] for i in range(refs)]

# needed for weak scaling
def make2d_N_alternating (refs, start=32):
  N = []
  tmp  = 0
  i_nx = 0
  i_ny = 0
  
  for ref in range(refs):
    
    N.append([start*2**i_nx,start*2**i_ny,1])
    
    if tmp == 0:
      i_nx += 1
      tmp = 1
    else:
      i_ny += 1
      tmp = 0
  
  return N
