
/* EQUATION SPECIFIC FVM routines
 * Linear Advection (scalar linear conservation law)
 */

///::title:: Linear Advection (scalar linear conservation law)
///::macro:: SPEED_HOOK @ 1.0

#include "equation.h"

void calc_point_flux(ConservedVars &F, const AllVariables &V, int d)
{
  F.u = SPEED_HOOK * V.U.u;
}

void apply_constraints (ConservedVars &CV, XtraVars &XV)
{
  // Constraints for ConservedVars
  /* nothing */
  // Constraints for XtraVars
  if (XV.E < 0.0) XV.E = 0.0;
}

real calc_time_step()
{
  return CFL * dx[X] / fabs(SPEED_HOOK);
}

void calc_xtra(XtraVars &Xijk, const ConservedVars &Uijk, int i, int j, int k)
{
  Xijk.E = Uijk.u * Uijk.u / 2.0;
}

void calc_all_vars(AllVariables &V, const PrimitiveVars &P)
{
  P.toConservedVars(V.U);
  V.XU.E = P.u * P.u / 2.0;
}

#if INDICATOR == 1
// u | needs to be reimplemented
real indicator(int i, int j, int k, int d)
{
  ConservedVars &Uijk = U[i][j][k];
  return Uijk.u;
}
#endif
