
/* Shallow Water equations
 * Structures and operator overloading for ConservedVars, PrimitiveVars and XtraVars
 */

#ifndef STRUCTURES_SW_H
#define STRUCTURES_SW_H

#include "numtypes.h"

// === ConservedVars

struct ConservedVars {
  real   h;
  vector hu;
  
  real norm() const
  {
    return fabs(h) + hu.norm();
  }
  
  // defined in "structures.cpp"
  
  void operator += (const ConservedVars &a);
  void operator -= (const ConservedVars &a);
  void operator *= (const ConservedVars &a);
  void operator /= (const ConservedVars &a);
  void operator /= (const real M);
  void operator *= (const real M);
  void operator =  (const real r);
  #if COMPUTE_KURTOSIS
  void operator -= (const real M);
  #endif
};

// === XtraVars

struct XtraVars {
  vector u;
  real   E;
  real   b;
  
  // defined in "structures.cpp"
  
  void operator += (const XtraVars &a);
  void operator -= (const XtraVars &a);
  void operator *= (const XtraVars &a);
  void operator /= (const XtraVars &a);
  void operator /= (const real r);
  void operator *= (const real r);
  void operator =  (const real r);
  #if COMPUTE_KURTOSIS
  void operator -= (const real r);
  #endif
};

// === PrimitiveVars

struct PrimitiveVars {
  real   h;
  vector u;
  
  void toConservedVars(ConservedVars &v) const {
    v.h = h;
    v.hu = h*u;
  }
  
  PrimitiveVars() {}
  PrimitiveVars(const ConservedVars &CV, const XtraVars &XV);
};

// picks PrimitiveVars from ConservedVars (U) and XtraVars (Xtra)
inline PrimitiveVars::PrimitiveVars(const ConservedVars &CV, const XtraVars &XV)
{
  h = CV.h;
  u = XV.u;
}

#endif
