
#include "main.h"
#include "multi.h"
#include "space.h"
#include "solver.h"

#if HLL_dt
  #include "hll_util.h"
#endif

// substep call for 3rd order, including quadrature integration.

void sub_step(bool firsttime)
{
  PRE_STEP_HOOK
  
  // calculate sources
  
  for (int i = 0; i < NXl; i++) {
    for (int j = 0; j < NYl; j++) {
      for (int k = 0; k < NZl; k++) {
        zero(Source[i][j][k]);
        SOURCE_HOOK
      }
    }
  }
  
  // calculate fluxes
  
  AllVariables lV, rV;
  vector us, Bs;
  
  #if HLL_dt
  vector maxeig;
  maxeig.zero();
  real sl, sr, cs;
  #endif
  
  for (int d = 0; d < NDIR; d++) {
    if (!HAVE_DIR(d)) continue;
    for (int i = 0; i < FLUX_NX(d); i++) {
      for (int j = 0; j < FLUX_NY(d); j++) {
        for (int k = 0; k < FLUX_NZ(d); k++) {
          
          right (lV, i-DX[d], j-DY[d], k-DZ[d], d);
          left  (rV, i, j, k, d);
          
          #if HLL_dt
          if (firsttime) {
            hll_speeds(lV, rV, d, sl, sr, cs);
            maxeig[d] = max(maxeig[d],max(fabs(sl),fabs(sr)));
          }
          #endif
          
          ConservedVars F;
          zero(Flux[d][i][j][k]);
          
          solver_calc_flux(F, us, Bs, lV, rV, i, j, k, d);
          
          Flux[d][i][j][k] += F;
          
          #if !NO_POWELL
          if (POWELL)            
            powell_source(us, Bs, lV, rV, i, j, k, d);
          #endif
          
          /*
          for (int p = 0; p < NQUAD; p++) {
            
            //right(lV, i-DX[d], j-DY[d], k-DZ[d], d, p);
            //left(rV, i, j, k, d, p);
            
            //Flux[d][i][j][k] += QUAD_WEIGHT[p]*F;
            Flux[d][i][j][k] += F;
            
            //powell_source(us, Bs, lV, rV, i, j, k, d, 1.0/NQUAD);
          }
          */
        }
      }
    }
  }
  
  // calculate time step
  
  #if HLL_dt
  if (firsttime) {
    dt = multi_dt(CFL/(maxeig[X]/dx[X] + maxeig[Y]/dx[Y] + maxeig[Z]/dx[Z]));
    if (t + dt > t_halt) dt = t_halt - t;
  }
  #endif
  
  // apply fluxes & sources

  for (int i = 0; i < NXl; i++) {
    for (int j = 0; j < NYl; j++) {
      for (int k = 0; k < NZl; k++) {
        for (int d = 0; d < NDIR; d++) {
          if (!HAVE_DIR(d)) continue;
          U[i][j][k] -= (dt/dx[d])*(Flux[d][i+DX[d]][j+DY[d]][k+DZ[d]] - Flux[d][i][j][k]);
        }
        U[i][j][k] += dt * Source[i][j][k];
      }
    }
  }
}
